@@ -11,13 +11,11 @@ class DynamicTableMap(ObjectMapper):
     def __init__(self, spec):
         super().__init__(spec)
         vector_data_spec = spec.get_data_type('VectorData')
-        vector_index_spec = spec.get_data_type('VectorIndex')
         self.map_spec('columns', vector_data_spec)
-        self.map_spec('columns', vector_index_spec)
 
     @ObjectMapper.object_attr('colnames')
     def attr_columns(self, container, manager):
-        if all(len(col) == 0 for col in container.columns):
+        if all(not col for col in container.columns):
             return tuple()
         return container.colnames
 
