@@ -145,7 +145,7 @@ class DataChunkIterator(AbstractDataChunkIterator):
                     self.__maxshape = self.data.shape
                     self.__data_iter = iter(self.data)
             # Try to get an accurate idea of __maxshape for other Python data structures if possible.
-            # Don't just callget_shape for a generator as that would potentially trigger loading of all the data
+            # Don't just call get_data_shape for a generator as that would potentially trigger loading of all the data
             elif isinstance(self.data, list) or isinstance(self.data, tuple):
                 self.__maxshape = get_data_shape(self.data, strict_no_data_load=True)
 
@@ -158,6 +158,10 @@ class DataChunkIterator(AbstractDataChunkIterator):
             self.__dtype = self.__next_chunk.data.dtype
             self.__first_chunk_shape = get_data_shape(self.__next_chunk.data)
 
+        # This should be done as a last resort only
+        if self.__first_chunk_shape is None and self.__maxshape is not None:
+            self.__first_chunk_shape = tuple(1 if i is None else i for i in self.__maxshape)
+
         if self.__dtype is None:
             raise Exception('Data type could not be determined. Please specify dtype in DataChunkIterator init.')
 
@@ -620,7 +624,11 @@ class DataIO:
         return len(self.data)
 
     def __bool__(self):
-        return self.valid and len(self) > 0
+        if self.valid:
+            if isinstance(self.data, AbstractDataChunkIterator):
+                return True
+            return len(self) > 0
+        return False
 
     def __getattr__(self, attr):
         """Delegate attribute lookup to data object"""
