@@ -36,6 +36,8 @@ class SpecNamespace(dict):
 
     __types_key = 'data_types'
 
+    UNVERSIONED = None  # value representing missing version
+
     @docval(*_namespace_args)
     def __init__(self, **kwargs):
         doc, full_name, name, version, date, author, contact, schema, catalog = \
@@ -48,8 +50,16 @@ class SpecNamespace(dict):
         self['name'] = name
         if full_name is not None:
             self['full_name'] = full_name
+        if version == str(SpecNamespace.UNVERSIONED):
+            # the unversioned version may be written to file as a string and read from file as a string
+            warn("Loaded namespace '%s' is unversioned. Please notify the extension author." % name)
+            version = SpecNamespace.UNVERSIONED
         if version is None:
-            raise TypeError('SpecNamespace missing arg `version`. Please specify a version for the extension.')
+            # version is required on write -- see YAMLSpecWriter.write_namespace -- but can be None on read in order to
+            # be able to read older files with extensions that are missing the version key.
+            warn(("Loaded namespace '%s' is missing the required key 'version'. Version will be set to '%s'. "
+                  "Please notify the extension author.") % (name, SpecNamespace.UNVERSIONED))
+            version = SpecNamespace.UNVERSIONED
         self['version'] = version
         if date is not None:
             self['date'] = date
@@ -79,13 +89,16 @@ class SpecNamespace(dict):
 
     @property
     def author(self):
-        """String or list of strings with the authors or  None"""
+        """String or list of strings with the authors or None"""
         return self.get('author', None)
 
     @property
     def version(self):
-        """String, list, or tuple with the version or None """
-        return self.get('version', None)
+        """
+        String, list, or tuple with the version or SpecNamespace.UNVERSIONED
+        if the version is missing or empty
+        """
+        return self.get('version', None) or SpecNamespace.UNVERSIONED
 
     @property
     def date(self):
@@ -422,7 +435,8 @@ class NamespaceCatalog:
                     catalog.register_spec(spec, spec_file)
                 included_types[s['namespace']] = tuple(types)
         # construct namespace
-        self.__namespaces[ns_name] = self.__spec_namespace_cls.build_namespace(catalog=catalog, **namespace)
+        ns = self.__spec_namespace_cls.build_namespace(catalog=catalog, **namespace)
+        self.__namespaces[ns_name] = ns
         return included_types
 
     @docval({'name': 'namespace_path', 'type': str, 'doc': 'the path to the file containing the namespaces(s) to load'},
