@@ -28,12 +28,8 @@ class Test1_1_0(TestCase):
         with HDF5IO(self.path_1_0_5, manager=self.manager, mode='r') as io:
             read_foofile = io.read()
             self.assertTrue(len(read_foofile.buckets) == 1)
-            # workaround for the fact that order of foos is not maintained
-            for foo in read_foofile.buckets[0].foos:
-                if foo.name == 'foo1':
-                    self.assertListEqual(foo.my_data[:].tolist(), [0, 1, 2, 3, 4])
-                if foo.name == 'foo2':
-                    self.assertListEqual(foo.my_data[:].tolist(), [5, 6, 7, 8, 9])
+            self.assertListEqual(read_foofile.buckets['test_bucket'].foos['foo1'].my_data[:].tolist(), [0, 1, 2, 3, 4])
+            self.assertListEqual(read_foofile.buckets['test_bucket'].foos['foo2'].my_data[:].tolist(), [5, 6, 7, 8, 9])
 
     def test_append_1_0_5(self):
         '''Test whether we can append to files made by hdmf version 1.0.5'''
@@ -42,13 +38,9 @@ class Test1_1_0(TestCase):
 
         with HDF5IO(self.path_1_0_5, manager=self.manager, mode='a') as io:
             read_foofile = io.read()
-            read_foofile.buckets.append(foobucket)
-            foobucket.parent = read_foofile
+            read_foofile.add_bucket(foobucket)
             io.write(read_foofile)
 
         with HDF5IO(self.path_1_0_5, manager=self.manager, mode='r') as io:
             read_foofile = io.read()
-            # workaround for the fact that order of buckets is not maintained
-            for bucket in read_foofile.buckets:
-                if bucket.name == 'foobucket2':
-                    self.assertListEqual(bucket.foos[0].my_data[:].tolist(), foo.my_data)
+            self.assertListEqual(read_foofile.buckets['foobucket2'].foos['foo3'].my_data[:].tolist(), foo.my_data)
