@@ -1,5 +1,6 @@
 from hdmf.spec import GroupSpec, AttributeSpec, DatasetSpec, SpecCatalog, SpecNamespace, NamespaceCatalog, RefSpec
-from hdmf.build import GroupBuilder, DatasetBuilder, ObjectMapper, BuildManager, TypeMap, LinkBuilder
+from hdmf.build import (GroupBuilder, DatasetBuilder, ObjectMapper, BuildManager, TypeMap, LinkBuilder,
+                        ReferenceBuilder, MissingRequiredWarning, OrphanContainerBuildError)
 from hdmf import Container
 from hdmf.utils import docval, getargs, get_docval
 from hdmf.data_utils import DataChunkIterator
@@ -8,6 +9,7 @@ from hdmf.testing import TestCase
 
 from abc import ABCMeta, abstractmethod
 import numpy as np
+import unittest
 
 from tests.unit.utils import CORE_NAMESPACE
 
@@ -63,6 +65,10 @@ class Bar(Container):
     def foo(self):
         return self.__foo
 
+    def remove_foo(self):
+        if self is self.__foo.parent:
+            self._remove_child(self.__foo)
+
 
 class Foo(Container):
 
@@ -119,11 +125,8 @@ class TestTypeMap(TestCase):
         self.type_map = TypeMap(self.namespace_catalog)
         self.type_map.register_container_type(CORE_NAMESPACE, 'Bar', Bar)
         self.type_map.register_container_type(CORE_NAMESPACE, 'Foo', Foo)
-        # self.build_manager = BuildManager(self.type_map)
 
     def test_get_map_unique_mappers(self):
-        self.type_map.register_map(Bar, ObjectMapper)
-        self.type_map.register_map(Foo, ObjectMapper)
         bar_inst = Bar('my_bar', list(range(10)), 'value1', 10)
         foo_inst = Foo(name='my_foo')
         bar_mapper = self.type_map.get_map(bar_inst)
@@ -131,7 +134,6 @@ class TestTypeMap(TestCase):
         self.assertIsNot(bar_mapper, foo_mapper)
 
     def test_get_map(self):
-        self.type_map.register_map(Bar, ObjectMapper)
         container_inst = Bar('my_bar', list(range(10)), 'value1', 10)
         mapper = self.type_map.get_map(container_inst)
         self.assertIsInstance(mapper, ObjectMapper)
@@ -229,7 +231,6 @@ class TestDynamicContainer(TestCase):
         self.namespace_catalog.add_namespace(CORE_NAMESPACE, self.namespace)
         self.type_map = TypeMap(self.namespace_catalog)
         self.type_map.register_container_type(CORE_NAMESPACE, 'Bar', Bar)
-        self.type_map.register_map(Bar, ObjectMapper)
         self.manager = BuildManager(self.type_map)
         self.mapper = ObjectMapper(self.bar_spec)
 
@@ -347,7 +348,7 @@ class TestDynamicContainer(TestCase):
         with self.assertRaises(TypeError):
             Baz1('My Baz', [1, 2, 3, 4], 'string attribute', 1000, attr3=98.6, attr4=1.0, baz2=bar)
 
-    def test_dynamic_container_composition_wrong_order(self):
+    def test_dynamic_container_composition_reverse_order(self):
         baz_spec2 = GroupSpec('A composition inside', data_type_def='Baz2',
                               data_type_inc=self.bar_spec,
                               attributes=[
@@ -360,13 +361,51 @@ class TestDynamicContainer(TestCase):
                               groups=[GroupSpec('A composition inside', data_type_inc='Baz2')])
         self.spec_catalog.register_spec(baz_spec1, 'extension.yaml')
         self.spec_catalog.register_spec(baz_spec2, 'extension.yaml')
+        Baz1 = self.type_map.get_container_cls(CORE_NAMESPACE, 'Baz1')
+        Baz2 = self.type_map.get_container_cls(CORE_NAMESPACE, 'Baz2')
+        Baz1('My Baz', [1, 2, 3, 4], 'string attribute', 1000, attr3=98.6, attr4=1.0,
+             baz2=Baz2('My Baz', [1, 2, 3, 4], 'string attribute', 1000, attr3=98.6, attr4=1.0))
 
-        # Setup all the data we need
-        msg = ("Cannot dynamically generate class for type 'Baz1'. Type 'Baz2' does not exist. "
-               "Please define that type before defining 'Baz1'.")
+        Bar = self.type_map.get_container_cls(CORE_NAMESPACE, 'Bar')
+        bar = Bar('My Bar', [1, 2, 3, 4], 'string attribute', 1000)
+
+        with self.assertRaises(TypeError):
+            Baz1('My Baz', [1, 2, 3, 4], 'string attribute', 1000, attr3=98.6, attr4=1.0, baz2=bar)
+
+    def test_dynamic_container_composition_missing_type(self):
+        baz_spec1 = GroupSpec('A composition test outside', data_type_def='Baz1', data_type_inc=self.bar_spec,
+                              attributes=[AttributeSpec('attr3', 'an example float attribute', 'float'),
+                                          AttributeSpec('attr4', 'another example float attribute', 'float')],
+                              groups=[GroupSpec('A composition inside', data_type_inc='Baz2')])
+        self.spec_catalog.register_spec(baz_spec1, 'extension.yaml')
+
+        msg = "No specification for 'Baz2' in namespace 'test_core'"
         with self.assertRaisesWith(ValueError, msg):
             self.manager.type_map.get_container_cls(CORE_NAMESPACE, 'Baz1')
 
+    def test_dynamic_container_uint(self):
+        baz_spec = GroupSpec('A test extension with no Container class',
+                             data_type_def='Baz', data_type_inc=self.bar_spec,
+                             attributes=[AttributeSpec('attr3', 'an example uint16 attribute', 'uint16'),
+                                         AttributeSpec('attr4', 'another example float attribute', 'float')])
+        self.spec_catalog.register_spec(baz_spec, 'extension.yaml')
+        cls = self.type_map.get_container_cls(CORE_NAMESPACE, 'Baz')
+        for arg in get_docval(cls.__init__):
+            if arg['name'] == 'attr3':
+                self.assertTupleEqual(arg['type'], (np.uint16, np.uint32, np.uint64))
+
+    def test_dynamic_container_numeric(self):
+        baz_spec = GroupSpec('A test extension with no Container class',
+                             data_type_def='Baz', data_type_inc=self.bar_spec,
+                             attributes=[AttributeSpec('attr3', 'an example numeric attribute', 'numeric'),
+                                         AttributeSpec('attr4', 'another example float attribute', 'float')])
+        self.spec_catalog.register_spec(baz_spec, 'extension.yaml')
+        cls = self.type_map.get_container_cls(CORE_NAMESPACE, 'Baz')
+        for arg in get_docval(cls.__init__):
+            if arg['name'] == 'attr3':
+                self.assertTupleEqual(arg['type'], (float, np.float32, np.float64, np.int8, np.int16, np.int32,
+                                                    np.int64, int, np.uint8, np.uint16, np.uint32, np.uint64))
+
 
 class ObjectMapperMixin(metaclass=ABCMeta):
 
@@ -382,7 +421,6 @@ class ObjectMapperMixin(metaclass=ABCMeta):
         self.namespace_catalog.add_namespace(CORE_NAMESPACE, self.namespace)
         self.type_map = TypeMap(self.namespace_catalog)
         self.type_map.register_container_type(CORE_NAMESPACE, 'Bar', Bar)
-        self.type_map.register_map(Bar, ObjectMapper)
         self.manager = BuildManager(self.type_map)
         self.mapper = ObjectMapper(self.bar_spec)
 
@@ -529,8 +567,6 @@ class TestLinkedContainer(TestCase):
         self.type_map = TypeMap(self.namespace_catalog)
         self.type_map.register_container_type(CORE_NAMESPACE, 'Foo', Foo)
         self.type_map.register_container_type(CORE_NAMESPACE, 'Bar', Bar)
-        self.type_map.register_map(Foo, ObjectMapper)
-        self.type_map.register_map(Bar, ObjectMapper)
         self.manager = BuildManager(self.type_map)
         self.foo_mapper = ObjectMapper(self.foo_spec)
         self.bar_mapper = ObjectMapper(self.bar_spec)
@@ -565,6 +601,192 @@ class TestLinkedContainer(TestCase):
         self.assertDictEqual(bar1_builder, bar1_expected)
         self.assertDictEqual(bar2_builder, bar2_expected)
 
+    @unittest.expectedFailure
+    def test_build_broken_link_parent(self):
+        ''' Test that building a container with a broken link that has a parent raises an error. '''
+        foo_inst = Foo('my_foo')
+        Bar('my_bar1', list(range(10)), 'value1', 10, foo=foo_inst)  # foo_inst.parent is this bar
+        # bar_inst2.foo should link to bar_inst1.foo
+        bar_inst2 = Bar('my_bar2', list(range(10)), 'value1', 10, foo=foo_inst)
+
+        # TODO bar_inst.foo.parent exists but is never built - this is a tricky edge case that should raise an error
+        with self.assertRaises(OrphanContainerBuildError):
+            self.bar_mapper.build(bar_inst2, self.manager)
+
+    def test_build_broken_link_no_parent(self):
+        ''' Test that building a container with a broken link that has no parent raises an error. '''
+        foo_inst = Foo('my_foo')
+        bar_inst1 = Bar('my_bar1', list(range(10)), 'value1', 10, foo=foo_inst)  # foo_inst.parent is this bar
+        # bar_inst2.foo should link to bar_inst1.foo
+        bar_inst2 = Bar('my_bar2', list(range(10)), 'value1', 10, foo=foo_inst)
+        bar_inst1.remove_foo()
+
+        msg = ("my_bar2 (my_bar2): Linked Foo 'my_foo' has no parent. Remove the link or ensure the linked container "
+               "is added properly.")
+        with self.assertRaisesWith(OrphanContainerBuildError, msg):
+            self.bar_mapper.build(bar_inst2, self.manager)
+
+
+class TestReference(TestCase):
+
+    def setUp(self):
+        self.foo_spec = GroupSpec('A test group specification with data type Foo', data_type_def='Foo')
+        self.bar_spec = GroupSpec('A test group specification with a data type Bar',
+                                  data_type_def='Bar',
+                                  datasets=[DatasetSpec('an example dataset', 'int', name='data')],
+                                  attributes=[AttributeSpec('attr1', 'an example string attribute', 'text'),
+                                              AttributeSpec('attr2', 'an example integer attribute', 'int'),
+                                              AttributeSpec('foo', 'a referenced foo', RefSpec('Foo', 'object'),
+                                                            required=False)])
+
+        self.spec_catalog = SpecCatalog()
+        self.spec_catalog.register_spec(self.foo_spec, 'test.yaml')
+        self.spec_catalog.register_spec(self.bar_spec, 'test.yaml')
+        self.namespace = SpecNamespace('a test namespace', CORE_NAMESPACE,
+                                       [{'source': 'test.yaml'}],
+                                       version='0.1.0',
+                                       catalog=self.spec_catalog)
+        self.namespace_catalog = NamespaceCatalog()
+        self.namespace_catalog.add_namespace(CORE_NAMESPACE, self.namespace)
+        self.type_map = TypeMap(self.namespace_catalog)
+        self.type_map.register_container_type(CORE_NAMESPACE, 'Foo', Foo)
+        self.type_map.register_container_type(CORE_NAMESPACE, 'Bar', Bar)
+        self.manager = BuildManager(self.type_map)
+        self.foo_mapper = ObjectMapper(self.foo_spec)
+        self.bar_mapper = ObjectMapper(self.bar_spec)
+
+    def test_build_attr_ref(self):
+        ''' Test default mapping functionality when one container contains an attribute reference to another container.
+        '''
+        foo_inst = Foo('my_foo')
+        bar_inst1 = Bar('my_bar1', list(range(10)), 'value1', 10, foo=foo_inst)
+        bar_inst2 = Bar('my_bar2', list(range(10)), 'value1', 10)
+
+        foo_builder = self.foo_mapper.build(foo_inst, self.manager)
+        bar1_builder = self.bar_mapper.build(bar_inst1, self.manager)
+        bar2_builder = self.bar_mapper.build(bar_inst2, self.manager)
+
+        foo_expected = GroupBuilder('my_foo')
+
+        inner_foo_builder = GroupBuilder('my_foo',
+                                         attributes={'data_type': 'Foo',
+                                                     'namespace': CORE_NAMESPACE,
+                                                     'object_id': foo_inst.object_id})
+        bar1_expected = GroupBuilder('n/a',  # name doesn't matter
+                                     datasets={'data': DatasetBuilder('data', list(range(10)))},
+                                     attributes={'attr1': 'value1',
+                                                 'attr2': 10,
+                                                 'foo': ReferenceBuilder(inner_foo_builder)})
+        bar2_expected = GroupBuilder('n/a',  # name doesn't matter
+                                     datasets={'data': DatasetBuilder('data', list(range(10)))},
+                                     attributes={'attr1': 'value1',
+                                                 'attr2': 10})
+        self.assertDictEqual(foo_builder, foo_expected)
+        self.assertDictEqual(bar1_builder, bar1_expected)
+        self.assertDictEqual(bar2_builder, bar2_expected)
+
+    def test_build_attr_ref_invalid(self):
+        ''' Test default mapping functionality when one container contains an attribute reference to another container.
+        '''
+        bar_inst1 = Bar('my_bar1', list(range(10)), 'value1', 10)
+        bar_inst1._Bar__foo = object()  # make foo object a non-container type
+
+        msg = "invalid type for reference 'foo' (<class 'object'>) - must be AbstractContainer"
+        with self.assertRaisesWith(ValueError, msg):
+            self.bar_mapper.build(bar_inst1, self.manager)
+
+
+class TestMissingRequiredAttribute(TestCase):
+
+    def test_required_attr_missing(self):
+        ''' Test mapping when one container is missing a required attribute
+        '''
+        bar_spec = GroupSpec('A test group specification with a data type Bar',
+                             data_type_def='Bar',
+                             datasets=[DatasetSpec('an example dataset', 'int', name='data')],
+                             attributes=[AttributeSpec('attr1', 'an example string attribute', 'text'),
+                                         AttributeSpec('attr2', 'an example integer attribute', 'int'),
+                                         AttributeSpec('foo', 'a referenced foo', RefSpec('Foo', 'object'))])
+
+        spec_catalog = SpecCatalog()
+        spec_catalog.register_spec(bar_spec, 'test.yaml')
+        namespace = SpecNamespace('a test namespace', CORE_NAMESPACE,
+                                  [{'source': 'test.yaml'}],
+                                  version='0.1.0',
+                                  catalog=spec_catalog)
+        namespace_catalog = NamespaceCatalog()
+        namespace_catalog.add_namespace(CORE_NAMESPACE, namespace)
+        type_map = TypeMap(namespace_catalog)
+        type_map.register_container_type(CORE_NAMESPACE, 'Bar', Bar)
+        manager = BuildManager(type_map)
+        bar_mapper = ObjectMapper(bar_spec)
+
+        bar_inst1 = Bar('my_bar1', list(range(10)), 'value1', 10)
+        bar_inst1._Bar__attr1 = None  # make attr1 attribute None
+
+        msg = "attribute 'attr1' for 'my_bar1' (Bar)"
+        with self.assertWarnsWith(MissingRequiredWarning, msg):
+            bar_mapper.build(bar_inst1, manager)
+
+    def test_required_attr_ref_missing(self):
+        ''' Test mapping when one container is missing a required attribute reference
+        '''
+        bar_spec = GroupSpec('A test group specification with a data type Bar',
+                             data_type_def='Bar',
+                             datasets=[DatasetSpec('an example dataset', 'int', name='data')],
+                             attributes=[AttributeSpec('attr1', 'an example string attribute', 'text'),
+                                         AttributeSpec('attr2', 'an example integer attribute', 'int'),
+                                         AttributeSpec('foo', 'a referenced foo', RefSpec('Foo', 'object'))])
+
+        spec_catalog = SpecCatalog()
+        spec_catalog.register_spec(bar_spec, 'test.yaml')
+        namespace = SpecNamespace('a test namespace', CORE_NAMESPACE,
+                                  [{'source': 'test.yaml'}],
+                                  version='0.1.0',
+                                  catalog=spec_catalog)
+        namespace_catalog = NamespaceCatalog()
+        namespace_catalog.add_namespace(CORE_NAMESPACE, namespace)
+        type_map = TypeMap(namespace_catalog)
+        type_map.register_container_type(CORE_NAMESPACE, 'Bar', Bar)
+        manager = BuildManager(type_map)
+        bar_mapper = ObjectMapper(bar_spec)
+
+        bar_inst1 = Bar('my_bar1', list(range(10)), 'value1', 10)
+
+        msg = "attribute 'foo' for 'my_bar1' (Bar)"
+        with self.assertWarnsWith(MissingRequiredWarning, msg):
+            bar_mapper.build(bar_inst1, manager)
+
+
+class TestMissingRequiredDataset(TestCase):
+
+    def test_required_dataset_missing(self):
+        ''' Test mapping when one container is missing a required dataset
+        '''
+        bar_spec = GroupSpec('A test group specification with a data type Bar',
+                             data_type_def='Bar',
+                             datasets=[DatasetSpec('an example dataset', 'int', name='data')])
+
+        spec_catalog = SpecCatalog()
+        spec_catalog.register_spec(bar_spec, 'test.yaml')
+        namespace = SpecNamespace('a test namespace', CORE_NAMESPACE,
+                                  [{'source': 'test.yaml'}],
+                                  version='0.1.0',
+                                  catalog=spec_catalog)
+        namespace_catalog = NamespaceCatalog()
+        namespace_catalog.add_namespace(CORE_NAMESPACE, namespace)
+        type_map = TypeMap(namespace_catalog)
+        type_map.register_container_type(CORE_NAMESPACE, 'Bar', Bar)
+        manager = BuildManager(type_map)
+        bar_mapper = ObjectMapper(bar_spec)
+
+        bar_inst1 = Bar('my_bar1', list(range(10)), 'value1', 10)
+        bar_inst1._Bar__data = None  # make data dataset None
+
+        msg = "dataset 'data' for 'my_bar1' (Bar)"
+        with self.assertWarnsWith(MissingRequiredWarning, msg):
+            bar_mapper.build(bar_inst1, manager)
+
 
 class TestConvertDtype(TestCase):
 
@@ -763,7 +985,8 @@ class TestConvertDtype(TestCase):
             with self.subTest(dtype=dtype):
                 s = np.dtype(self._get_type(spec_type))
                 g = np.dtype(self._get_type(dtype))
-                msg = 'Value with data type %s is being converted to data type %s as specified.' % (g.name, s.name)
+                msg = ("Spec 'data': Value with data type %s is being converted to data type %s as specified."
+                       % (g.name, s.name))
                 with self.assertWarnsWith(UserWarning, msg):
                     ret = ObjectMapper.convert_dtype(spec, value)
                 self.assertTupleEqual(ret, match)
@@ -790,8 +1013,8 @@ class TestConvertDtype(TestCase):
                 s = np.dtype(self._get_type(spec_type))
                 e = np.dtype(self._get_type(exp_type))
                 g = np.dtype(self._get_type(dtype))
-                msg = ('Value with data type %s is being converted to data type %s (min specification: %s).'
-                       % (g.name, e.name, s.name))
+                msg = ("Spec 'data': Value with data type %s is being converted to data type %s "
+                       "(min specification: %s)." % (g.name, e.name, s.name))
                 with self.assertWarnsWith(UserWarning, msg):
                     ret = ObjectMapper.convert_dtype(spec, value)
                 self.assertTupleEqual(ret, match)
@@ -809,86 +1032,200 @@ class TestConvertDtype(TestCase):
                 with self.assertRaisesWith(ValueError, msg):
                     ObjectMapper.convert_dtype(spec, value)
 
+    def test_text_spec(self):
+        spec_type = 'text'
+        spec = DatasetSpec('an example dataset', spec_type, name='data')
+
+        value = 'a'
+        ret, ret_dtype = ObjectMapper.convert_dtype(spec, value)
+        self.assertEqual(ret, value)
+        self.assertIs(type(ret), str)
+        self.assertEqual(ret_dtype, 'utf8')
+
+        value = b'a'
+        ret, ret_dtype = ObjectMapper.convert_dtype(spec, value)
+        self.assertEqual(ret, 'a')
+        self.assertIs(type(ret), str)
+        self.assertEqual(ret_dtype, 'utf8')
+
+        value = ['a', 'b']
+        ret, ret_dtype = ObjectMapper.convert_dtype(spec, value)
+        self.assertListEqual(ret, value)
+        self.assertIs(type(ret[0]), str)
+        self.assertEqual(ret_dtype, 'utf8')
+
+        value = np.array(['a', 'b'])
+        ret, ret_dtype = ObjectMapper.convert_dtype(spec, value)
+        np.testing.assert_array_equal(ret, value)
+        self.assertEqual(ret_dtype, 'utf8')
+
+        value = np.array(['a', 'b'], dtype='S1')
+        ret, ret_dtype = ObjectMapper.convert_dtype(spec, value)
+        np.testing.assert_array_equal(ret, np.array(['a', 'b'], dtype='U1'))
+        self.assertEqual(ret_dtype, 'utf8')
+
+    def test_ascii_spec(self):
+        spec_type = 'ascii'
+        spec = DatasetSpec('an example dataset', spec_type, name='data')
+
+        value = 'a'
+        ret, ret_dtype = ObjectMapper.convert_dtype(spec, value)
+        self.assertEqual(ret, b'a')
+        self.assertIs(type(ret), bytes)
+        self.assertEqual(ret_dtype, 'ascii')
+
+        value = b'a'
+        ret, ret_dtype = ObjectMapper.convert_dtype(spec, value)
+        self.assertEqual(ret, b'a')
+        self.assertIs(type(ret), bytes)
+        self.assertEqual(ret_dtype, 'ascii')
+
+        value = ['a', 'b']
+        ret, ret_dtype = ObjectMapper.convert_dtype(spec, value)
+        self.assertListEqual(ret, [b'a', b'b'])
+        self.assertIs(type(ret[0]), bytes)
+        self.assertEqual(ret_dtype, 'ascii')
+
+        value = np.array(['a', 'b'])
+        ret, ret_dtype = ObjectMapper.convert_dtype(spec, value)
+        np.testing.assert_array_equal(ret, np.array(['a', 'b'], dtype='S1'))
+        self.assertEqual(ret_dtype, 'ascii')
+
+        value = np.array(['a', 'b'], dtype='S1')
+        ret, ret_dtype = ObjectMapper.convert_dtype(spec, value)
+        np.testing.assert_array_equal(ret, value)
+        self.assertEqual(ret_dtype, 'ascii')
+
     def test_no_spec(self):
         spec_type = None
         spec = DatasetSpec('an example dataset', spec_type, name='data')
 
         value = [1, 2, 3]
-        ret = ObjectMapper.convert_dtype(spec, value)
-        match = (value, int)
-        self.assertTupleEqual(ret, match)
-        self.assertIs(type(ret[0][0]), match[1])
+        ret, ret_dtype = ObjectMapper.convert_dtype(spec, value)
+        self.assertListEqual(ret, value)
+        self.assertIs(type(ret[0]), int)
+        self.assertEqual(ret_dtype, int)
 
         value = np.uint64(4)
-        ret = ObjectMapper.convert_dtype(spec, value)
-        match = (value, np.uint64)
-        self.assertTupleEqual(ret, match)
-        self.assertIs(type(ret[0]), match[1])
+        ret, ret_dtype = ObjectMapper.convert_dtype(spec, value)
+        self.assertEqual(ret, value)
+        self.assertIs(type(ret), np.uint64)
+        self.assertEqual(ret_dtype, np.uint64)
 
         value = 'hello'
-        ret = ObjectMapper.convert_dtype(spec, value)
-        match = (value, 'utf8')
-        self.assertTupleEqual(ret, match)
-        self.assertIs(type(ret[0]), str)
+        ret, ret_dtype = ObjectMapper.convert_dtype(spec, value)
+        self.assertEqual(ret, value)
+        self.assertIs(type(ret), str)
+        self.assertEqual(ret_dtype, 'utf8')
 
-        value = bytes('hello', encoding='utf-8')
-        ret = ObjectMapper.convert_dtype(spec, value)
-        match = (value, 'ascii')
-        self.assertTupleEqual(ret, match)
-        self.assertIs(type(ret[0]), bytes)
+        value = b'hello'
+        ret, ret_dtype = ObjectMapper.convert_dtype(spec, value)
+        self.assertEqual(ret, value)
+        self.assertIs(type(ret), bytes)
+        self.assertEqual(ret_dtype, 'ascii')
+
+        value = np.array(['aa', 'bb'])
+        ret, ret_dtype = ObjectMapper.convert_dtype(spec, value)
+        np.testing.assert_array_equal(ret, value)
+        self.assertEqual(ret_dtype, 'utf8')
+
+        value = np.array(['aa', 'bb'], dtype='S2')
+        ret, ret_dtype = ObjectMapper.convert_dtype(spec, value)
+        np.testing.assert_array_equal(ret, value)
+        self.assertEqual(ret_dtype, 'ascii')
 
         value = DataChunkIterator(data=[1, 2, 3])
-        ret = ObjectMapper.convert_dtype(spec, value)
-        match = (value, np.dtype(int).type)
-        self.assertTupleEqual(ret, match)
-        self.assertIs(ret[0].dtype.type, match[1])
+        ret, ret_dtype = ObjectMapper.convert_dtype(spec, value)
+        self.assertEqual(ret, value)
+        self.assertIs(ret.dtype.type, np.dtype(int).type)
+        self.assertIs(type(ret.data[0]), int)
+        self.assertEqual(ret_dtype, np.dtype(int).type)
 
-        value = DataChunkIterator(data=[1., 2., 3.])
-        ret = ObjectMapper.convert_dtype(spec, value)
-        match = (value, np.dtype(float).type)
-        self.assertTupleEqual(ret, match)
-        self.assertIs(ret[0].dtype.type, match[1])
+        value = DataChunkIterator(data=['a', 'b'])
+        ret, ret_dtype = ObjectMapper.convert_dtype(spec, value)
+        self.assertEqual(ret, value)
+        self.assertIs(ret.dtype.type, np.str_)
+        self.assertIs(type(ret.data[0]), str)
+        self.assertEqual(ret_dtype, 'utf8')
 
         value = H5DataIO(np.arange(30).reshape(5, 2, 3))
-        ret = ObjectMapper.convert_dtype(spec, value)
-        match = (value, np.dtype(int).type)
-        self.assertTupleEqual(ret, match)
-        self.assertIs(ret[0].dtype.type, match[1])
+        ret, ret_dtype = ObjectMapper.convert_dtype(spec, value)
+        self.assertEqual(ret, value)
+        self.assertIs(ret.data.dtype.type, np.dtype(int).type)
+        self.assertEqual(ret_dtype, np.dtype(int).type)
 
-        value = H5DataIO(['foo' 'bar'])
-        ret = ObjectMapper.convert_dtype(spec, value)
-        match = (value, 'utf8')
-        self.assertTupleEqual(ret, match)
-        self.assertIs(type(ret[0].data[0]), str)
+        value = H5DataIO(['foo', 'bar'])
+        ret, ret_dtype = ObjectMapper.convert_dtype(spec, value)
+        self.assertEqual(ret, value)
+        self.assertIs(type(ret.data[0]), str)
+        self.assertEqual(ret_dtype, 'utf8')
+
+        value = H5DataIO([b'foo', b'bar'])
+        ret, ret_dtype = ObjectMapper.convert_dtype(spec, value)
+        self.assertEqual(ret, value)
+        self.assertIs(type(ret.data[0]), bytes)
+        self.assertEqual(ret_dtype, 'ascii')
 
     def test_numeric_spec(self):
         spec_type = 'numeric'
         spec = DatasetSpec('an example dataset', spec_type, name='data')
 
         value = np.uint64(4)
-        ret = ObjectMapper.convert_dtype(spec, value)
-        match = (value, np.uint64)
-        self.assertTupleEqual(ret, match)
-        self.assertIs(type(ret[0]), match[1])
+        ret, ret_dtype = ObjectMapper.convert_dtype(spec, value)
+        self.assertEqual(ret, value)
+        self.assertIs(type(ret), np.uint64)
+        self.assertEqual(ret_dtype, np.uint64)
 
         value = DataChunkIterator(data=[1, 2, 3])
-        ret = ObjectMapper.convert_dtype(spec, value)
-        match = (value, np.dtype(int).type)
-        self.assertTupleEqual(ret, match)
-        self.assertIs(ret[0].dtype.type, match[1])
+        ret, ret_dtype = ObjectMapper.convert_dtype(spec, value)
+        self.assertEqual(ret, value)
+        self.assertIs(ret.dtype.type, np.dtype(int).type)
+        self.assertIs(type(ret.data[0]), int)
+        self.assertEqual(ret_dtype, np.dtype(int).type)
+
+        value = ['a', 'b']
+        msg = "Cannot convert from <class 'str'> to 'numeric' specification dtype."
+        with self.assertRaisesWith(ValueError, msg):
+            ObjectMapper.convert_dtype(spec, value)
+
+        value = np.array(['a', 'b'])
+        msg = "Cannot convert from <class 'numpy.str_'> to 'numeric' specification dtype."
+        with self.assertRaisesWith(ValueError, msg):
+            ObjectMapper.convert_dtype(spec, value)
 
     def test_bool_spec(self):
         spec_type = 'bool'
         spec = DatasetSpec('an example dataset', spec_type, name='data')
 
         value = np.bool_(True)
-        ret = ObjectMapper.convert_dtype(spec, value)
-        match = (value, np.bool_)
-        self.assertTupleEqual(ret, match)
-        self.assertIs(type(ret[0]), match[1])
+        ret, ret_dtype = ObjectMapper.convert_dtype(spec, value)
+        self.assertEqual(ret, value)
+        self.assertIs(type(ret), np.bool_)
+        self.assertEqual(ret_dtype, np.bool_)
 
         value = True
-        ret = ObjectMapper.convert_dtype(spec, value)
-        match = (value, np.bool_)
-        self.assertTupleEqual(ret, match)
-        self.assertIs(type(ret[0]), match[1])
+        ret, ret_dtype = ObjectMapper.convert_dtype(spec, value)
+        self.assertEqual(ret, value)
+        self.assertIs(type(ret), np.bool_)
+        self.assertEqual(ret_dtype, np.bool_)
+
+    def test_override_type_int_restrict_precision(self):
+        spec = DatasetSpec('an example dataset', 'int8', name='data')
+        res = ObjectMapper.convert_dtype(spec, np.int64(1), 'int64')
+        self.assertTupleEqual(res, (np.int64(1), np.int64))
+
+    def test_override_type_numeric_to_uint(self):
+        spec = DatasetSpec('an example dataset', 'numeric', name='data')
+        res = ObjectMapper.convert_dtype(spec, np.uint32(1), 'uint8')
+        self.assertTupleEqual(res, (np.uint32(1), np.uint32))
+
+    def test_override_type_numeric_to_uint_list(self):
+        spec = DatasetSpec('an example dataset', 'numeric', name='data')
+        res = ObjectMapper.convert_dtype(spec, np.uint32((1, 2, 3)), 'uint8')
+        np.testing.assert_array_equal(res[0], np.uint32((1, 2, 3)))
+        self.assertEqual(res[1], np.uint32)
+
+    def test_override_type_none_to_bool(self):
+        spec = DatasetSpec('an example dataset', None, name='data')
+        res = ObjectMapper.convert_dtype(spec, True, 'bool')
+        self.assertTupleEqual(res, (True, np.bool_))
