@@ -1,6 +1,6 @@
 import numpy as np
 
-from hdmf.utils import docval, fmt_docval_args, get_docval, popargs, AllowPositional
+from hdmf.utils import docval, fmt_docval_args, get_docval, getargs, popargs, AllowPositional
 from hdmf.testing import TestCase
 
 
@@ -46,7 +46,7 @@ class MyTestSubclass(MyTestClass):
             {'name': 'arg3', 'type': bool, 'doc': 'argument3 is a bool. it defaults to False', 'default': False},
             {'name': 'arg4', 'type': str, 'doc': 'argument4 is a str'},
             {'name': 'arg5', 'type': 'float', 'doc': 'argument5 is a float'},
-            {'name': 'arg6', 'type': bool, 'doc': 'argument6 is a bool. it defaults to False', 'default': None})
+            {'name': 'arg6', 'type': bool, 'doc': 'argument6 is a bool. it defaults to None', 'default': None})
     def basic_add2_kw(self, **kwargs):
         return kwargs
 
@@ -107,7 +107,7 @@ class TestDocValidator(TestCase):
         self.test_obj_sub = MyTestSubclass()
 
     def test_bad_type(self):
-        exp_msg = (r"error parsing 'arg1' argument' : argtype must be a type, "
+        exp_msg = (r"docval for arg1: error parsing argument type: argtype must be a type, "
                    r"a str, a list, a tuple, or None - got <class|type 'dict'>")
         with self.assertRaisesRegex(Exception, exp_msg):
             @docval({'name': 'arg1', 'type': {'a': 1}, 'doc': 'this is a bad type'})
@@ -434,11 +434,11 @@ class TestDocValidator(TestCase):
         """Test that docval does not allow setting of arguments
            marked as unsupported
         """
-        @docval({'name': 'arg1', 'type': 'array_data', 'doc': 'this is a bad shape', 'unsupported': 'hi!'})
-        def method(self, **kwargs):
-            pass
-        with self.assertRaises(ValueError):
-            method(self, arg1=[[1, 1]])
+        msg = "docval for arg1: keys ['unsupported'] are not supported by docval"
+        with self.assertRaisesWith(Exception, msg):
+            @docval({'name': 'arg1', 'type': 'array_data', 'doc': 'this is a bad shape', 'unsupported': 'hi!'})
+            def method(self, **kwargs):
+                pass
 
     def test_catch_dup_names(self):
         """Test that docval does not allow duplicate argument names
@@ -527,6 +527,52 @@ class TestDocValidator(TestCase):
         self.assertEqual(res, np.bool_(True))
         self.assertIsInstance(res, np.bool_)
 
+    def test_uint_type(self):
+        """Test that docval type specification of np.uint32 works as expected."""
+        @docval({'name': 'arg1', 'type': np.uint32, 'doc': 'this is a uint'})
+        def method(self, **kwargs):
+            return popargs('arg1', kwargs)
+
+        res = method(self, arg1=np.uint32(1))
+        self.assertEqual(res, np.uint32(1))
+        self.assertIsInstance(res, np.uint32)
+
+        msg = ("TestDocValidator.test_uint_type.<locals>.method: incorrect type for 'arg1' (got 'uint8', expected "
+               "'uint32')")
+        with self.assertRaisesWith(TypeError, msg):
+            method(self, arg1=np.uint8(1))
+
+        msg = ("TestDocValidator.test_uint_type.<locals>.method: incorrect type for 'arg1' (got 'uint64', expected "
+               "'uint32')")
+        with self.assertRaisesWith(TypeError, msg):
+            method(self, arg1=np.uint64(1))
+
+    def test_uint_string_type(self):
+        """Test that docval type specification of string 'uint' matches np.uint of all available precisions."""
+        @docval({'name': 'arg1', 'type': 'uint', 'doc': 'this is a uint'})
+        def method(self, **kwargs):
+            return popargs('arg1', kwargs)
+
+        res = method(self, arg1=np.uint(1))
+        self.assertEqual(res, np.uint(1))
+        self.assertIsInstance(res, np.uint)
+
+        res = method(self, arg1=np.uint8(1))
+        self.assertEqual(res, np.uint8(1))
+        self.assertIsInstance(res, np.uint8)
+
+        res = method(self, arg1=np.uint16(1))
+        self.assertEqual(res, np.uint16(1))
+        self.assertIsInstance(res, np.uint16)
+
+        res = method(self, arg1=np.uint32(1))
+        self.assertEqual(res, np.uint32(1))
+        self.assertIsInstance(res, np.uint32)
+
+        res = method(self, arg1=np.uint64(1))
+        self.assertEqual(res, np.uint64(1))
+        self.assertIsInstance(res, np.uint64)
+
     def test_allow_positional_warn(self):
         @docval({'name': 'arg1', 'type': bool, 'doc': 'this is a bool'}, allow_positional=AllowPositional.WARNING)
         def method(self, **kwargs):
@@ -559,6 +605,134 @@ class TestDocValidator(TestCase):
         with self.assertRaisesWith(SyntaxError, msg):
             method(self, True)
 
+    def test_enum_str(self):
+        """Test that the basic usage of an enum check on strings works"""
+        @docval({'name': 'arg1', 'type': str, 'doc': 'an arg', 'enum': ['a', 'b']})  # also use enum: list
+        def method(self, **kwargs):
+            return popargs('arg1', kwargs)
+
+        self.assertEqual(method(self, 'a'), 'a')
+        self.assertEqual(method(self, 'b'), 'b')
+
+        msg = ("TestDocValidator.test_enum_str.<locals>.method: "
+               "forbidden value for 'arg1' (got 'c', expected ['a', 'b'])")
+        with self.assertRaisesWith(ValueError, msg):
+            method(self, 'c')
+
+    def test_enum_int(self):
+        """Test that the basic usage of an enum check on ints works"""
+        @docval({'name': 'arg1', 'type': int, 'doc': 'an arg', 'enum': (1, 2)})
+        def method(self, **kwargs):
+            return popargs('arg1', kwargs)
+
+        self.assertEqual(method(self, 1), 1)
+        self.assertEqual(method(self, 2), 2)
+
+        msg = ("TestDocValidator.test_enum_int.<locals>.method: "
+               "forbidden value for 'arg1' (got 3, expected (1, 2))")
+        with self.assertRaisesWith(ValueError, msg):
+            method(self, 3)
+
+    def test_enum_uint(self):
+        """Test that the basic usage of an enum check on uints works"""
+        @docval({'name': 'arg1', 'type': np.uint, 'doc': 'an arg', 'enum': (np.uint(1), np.uint(2))})
+        def method(self, **kwargs):
+            return popargs('arg1', kwargs)
+
+        self.assertEqual(method(self, np.uint(1)), np.uint(1))
+        self.assertEqual(method(self, np.uint(2)), np.uint(2))
+
+        msg = ("TestDocValidator.test_enum_uint.<locals>.method: "
+               "forbidden value for 'arg1' (got 3, expected (1, 2))")
+        with self.assertRaisesWith(ValueError, msg):
+            method(self, np.uint(3))
+
+    def test_enum_float(self):
+        """Test that the basic usage of an enum check on floats works"""
+        @docval({'name': 'arg1', 'type': float, 'doc': 'an arg', 'enum': (3.14, )})
+        def method(self, **kwargs):
+            return popargs('arg1', kwargs)
+
+        self.assertEqual(method(self, 3.14), 3.14)
+
+        msg = ("TestDocValidator.test_enum_float.<locals>.method: "
+               "forbidden value for 'arg1' (got 3.0, expected (3.14,))")
+        with self.assertRaisesWith(ValueError, msg):
+            method(self, 3.)
+
+    def test_enum_bool_mixed(self):
+        """Test that the basic usage of an enum check on a tuple of bool, int, float, and string works"""
+        @docval({'name': 'arg1', 'type': (bool, int, float, str, np.uint), 'doc': 'an arg',
+                 'enum': (True, 1, 1.0, 'true', np.uint(1))})
+        def method(self, **kwargs):
+            return popargs('arg1', kwargs)
+
+        self.assertEqual(method(self, True), True)
+        self.assertEqual(method(self, 1), 1)
+        self.assertEqual(method(self, 1.0), 1.0)
+        self.assertEqual(method(self, 'true'), 'true')
+        self.assertEqual(method(self, np.uint(1)), np.uint(1))
+
+        msg = ("TestDocValidator.test_enum_bool_mixed.<locals>.method: "
+               "forbidden value for 'arg1' (got 0, expected (True, 1, 1.0, 'true', 1))")
+        with self.assertRaisesWith(ValueError, msg):
+            method(self, 0)
+
+    def test_enum_bad_type(self):
+        """Test that docval with an enum check where the arg type includes an invalid enum type fails"""
+        msg = ("docval for arg1: enum checking cannot be used with arg type (<class 'bool'>, <class 'int'>, "
+               "<class 'str'>, <class 'numpy.float64'>, <class 'object'>)")
+        with self.assertRaisesWith(Exception, msg):
+            @docval({'name': 'arg1', 'type': (bool, int, str, np.float64, object), 'doc': 'an arg', 'enum': (1, 2)})
+            def method(self, **kwargs):
+                return popargs('arg1', kwargs)
+
+    def test_enum_none_type(self):
+        """Test that the basic usage of an enum check on None works"""
+        msg = ("docval for arg1: enum checking cannot be used with arg type None")
+        with self.assertRaisesWith(Exception, msg):
+            @docval({'name': 'arg1', 'type': None, 'doc': 'an arg', 'enum': (True, 1, 'true')})
+            def method(self, **kwargs):
+                pass
+
+    def test_enum_single_allowed(self):
+        """Test that docval with an enum check on a single value fails"""
+        msg = ("docval for arg1: enum value must be a list or tuple (received <class 'str'>)")
+        with self.assertRaisesWith(Exception, msg):
+            @docval({'name': 'arg1', 'type': str, 'doc': 'an arg', 'enum': 'only one value'})
+            def method(self, **kwargs):
+                pass
+
+    def test_enum_str_default(self):
+        """Test that docval with an enum check on strings and a default value works"""
+        @docval({'name': 'arg1', 'type': str, 'doc': 'an arg', 'default': 'a', 'enum': ['a', 'b']})
+        def method(self, **kwargs):
+            return popargs('arg1', kwargs)
+
+        self.assertEqual(method(self), 'a')
+
+        msg = ("TestDocValidator.test_enum_str_default.<locals>.method: "
+               "forbidden value for 'arg1' (got 'c', expected ['a', 'b'])")
+        with self.assertRaisesWith(ValueError, msg):
+            method(self, 'c')
+
+    def test_enum_str_none_default(self):
+        """Test that docval with an enum check on strings and a None default value works"""
+        @docval({'name': 'arg1', 'type': str, 'doc': 'an arg', 'default': None, 'enum': ['a', 'b']})
+        def method(self, **kwargs):
+            return popargs('arg1', kwargs)
+
+        self.assertIsNone(method(self))
+
+    def test_enum_forbidden_values(self):
+        """Test that docval with enum values that include a forbidden type fails"""
+        msg = ("docval for arg1: enum values are of types not allowed by arg type "
+               "(got [<class 'bool'>, <class 'list'>], expected <class 'bool'>)")
+        with self.assertRaisesWith(Exception, msg):
+            @docval({'name': 'arg1', 'type': bool, 'doc': 'an arg', 'enum': (True, [])})
+            def method(self, **kwargs):
+                pass
+
 
 class TestDocValidatorChain(TestCase):
 
@@ -654,3 +828,143 @@ class TestDocValidatorChain(TestCase):
                    r"\(expected shape '\(None, 2\)'\)")
         with self.assertRaisesRegex(ValueError, err_msg):
             MyChainClass(self.obj1, [[100, 200], [300, 400], [500, 600]], arg4=obj2)
+
+
+class TestGetargs(TestCase):
+    """Test the getargs function and its error conditions."""
+
+    def test_one_arg_first(self):
+        kwargs = {'a': 1, 'b': None}
+        expected_kwargs = kwargs.copy()
+        res = getargs('a', kwargs)
+        self.assertEqual(res, 1)
+        self.assertDictEqual(kwargs, expected_kwargs)
+
+    def test_one_arg_second(self):
+        kwargs = {'a': 1, 'b': None}
+        expected_kwargs = kwargs.copy()
+        res = getargs('b', kwargs)
+        self.assertEqual(res, None)
+        self.assertDictEqual(kwargs, expected_kwargs)
+
+    def test_many_args_get_some(self):
+        kwargs = {'a': 1, 'b': None, 'c': 3}
+        expected_kwargs = kwargs.copy()
+
+        res = getargs('a', 'c', kwargs)
+        self.assertListEqual(res, [1, 3])
+        self.assertDictEqual(kwargs, expected_kwargs)
+
+    def test_many_args_get_all(self):
+        kwargs = {'a': 1, 'b': None, 'c': 3}
+        expected_kwargs = kwargs.copy()
+
+        res = getargs('a', 'b', 'c', kwargs)
+        self.assertListEqual(res, [1, None, 3])
+        self.assertDictEqual(kwargs, expected_kwargs)
+
+    def test_many_args_reverse(self):
+        kwargs = {'a': 1, 'b': None, 'c': 3}
+        expected_kwargs = kwargs.copy()
+        res = getargs('c', 'b', 'a', kwargs)
+        self.assertListEqual(res, [3, None, 1])
+        self.assertDictEqual(kwargs, expected_kwargs)
+
+    def test_many_args_unpack(self):
+        kwargs = {'a': 1, 'b': None, 'c': 3}
+        expected_kwargs = kwargs.copy()
+        res1, res2, res3 = getargs('a', 'b', 'c', kwargs)
+        self.assertEqual(res1, 1)
+        self.assertEqual(res2, None)
+        self.assertEqual(res3, 3)
+        self.assertDictEqual(kwargs, expected_kwargs)
+
+    def test_too_few_args(self):
+        kwargs = {'a': 1, 'b': None}
+        msg = 'Must supply at least one key and a dict'
+        with self.assertRaisesWith(ValueError, msg):
+            getargs(kwargs)
+
+    def test_last_arg_not_dict(self):
+        kwargs = {'a': 1, 'b': None}
+        msg = 'Last argument must be a dict'
+        with self.assertRaisesWith(ValueError, msg):
+            getargs(kwargs, 'a')
+
+    def test_arg_not_found_one_arg(self):
+        kwargs = {'a': 1, 'b': None}
+        msg = "Argument not found in dict: 'c'"
+        with self.assertRaisesWith(ValueError, msg):
+            getargs('c', kwargs)
+
+    def test_arg_not_found_many_args(self):
+        kwargs = {'a': 1, 'b': None}
+        msg = "Argument not found in dict: 'c'"
+        with self.assertRaisesWith(ValueError, msg):
+            getargs('a', 'c', kwargs)
+
+
+class TestPopargs(TestCase):
+    """Test the popargs function and its error conditions."""
+
+    def test_one_arg_first(self):
+        kwargs = {'a': 1, 'b': None}
+        res = popargs('a', kwargs)
+        self.assertEqual(res, 1)
+        self.assertDictEqual(kwargs, {'b': None})
+
+    def test_one_arg_second(self):
+        kwargs = {'a': 1, 'b': None}
+        res = popargs('b', kwargs)
+        self.assertEqual(res, None)
+        self.assertDictEqual(kwargs, {'a': 1})
+
+    def test_many_args_pop_some(self):
+        kwargs = {'a': 1, 'b': None, 'c': 3}
+        res = popargs('a', 'c', kwargs)
+        self.assertListEqual(res, [1, 3])
+        self.assertDictEqual(kwargs, {'b': None})
+
+    def test_many_args_pop_all(self):
+        kwargs = {'a': 1, 'b': None, 'c': 3}
+        res = popargs('a', 'b', 'c', kwargs)
+        self.assertListEqual(res, [1, None, 3])
+        self.assertDictEqual(kwargs, {})
+
+    def test_many_args_reverse(self):
+        kwargs = {'a': 1, 'b': None, 'c': 3}
+        res = popargs('c', 'b', 'a', kwargs)
+        self.assertListEqual(res, [3, None, 1])
+        self.assertDictEqual(kwargs, {})
+
+    def test_many_args_unpack(self):
+        kwargs = {'a': 1, 'b': None, 'c': 3}
+        res1, res2, res3 = popargs('a', 'b', 'c', kwargs)
+        self.assertEqual(res1, 1)
+        self.assertEqual(res2, None)
+        self.assertEqual(res3, 3)
+        self.assertDictEqual(kwargs, {})
+
+    def test_too_few_args(self):
+        kwargs = {'a': 1, 'b': None}
+        msg = 'Must supply at least one key and a dict'
+        with self.assertRaisesWith(ValueError, msg):
+            popargs(kwargs)
+
+    def test_last_arg_not_dict(self):
+        kwargs = {'a': 1, 'b': None}
+        msg = 'Last argument must be a dict'
+        with self.assertRaisesWith(ValueError, msg):
+            popargs(kwargs, 'a')
+
+    def test_arg_not_found_one_arg(self):
+        kwargs = {'a': 1, 'b': None}
+        msg = "Argument not found in dict: 'c'"
+        with self.assertRaisesWith(ValueError, msg):
+            popargs('c', kwargs)
+
+    def test_arg_not_found_many_args(self):
+        kwargs = {'a': 1, 'b': None}
+        msg = "Argument not found in dict: 'c'"
+        with self.assertRaisesWith(ValueError, msg):
+            popargs('a', 'c', kwargs)
